//Importing d3 library and scatterplot library for our visualization.
import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';


//Defining a url for fetching the iris dataset uploaded in a gist.
//The url is defined in elements of an array form and is joined used .join function.
//The .join function is taking in an empty delimiter.
const csvUrl = [
  'https://gist.githubusercontent.com/',
  'ubl2/', // User
  '0999e3b16124edd5847795977eaa4547/', // Id of the Gist
  'raw/1ac148267b75ee7d22a1e16de0d543f935ddd60b/', // commit
  'Iris.csv', // File name
].join('');

//Function to parse through the columns and return the variable d with the information regarding each datapoint.
const parseRow = (d) => {
  d.sepal_length = +d.sepal_length;
  d.sepal_width = +d.sepal_width;
  d.petal_length = +d.petal_length;
  d.petal_width = +d.petal_width;
  return d;
};


//Defining variabled height and width based on the window side. This makes the graph scalable to the screen size.
const width = window.innerWidth;
const height = window.innerHeight;

//Defining the basic plot svg.
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

//Defining the attributes of the dataset and integrating them to the graph dimensions.
const xValue = (d) => d.petal_length;
const yValue = (d) => d.petal_width;
const rxValue = (d) => d.sepal_length;
const ryValue = (d) => d.sepal_width;
const speciesValue = (d) => d.species;

//Defining a main function for plotting the dynamic graph.
const main = async () => {
//Fetching the data and parsing trough the data.
const data = await csv(csvUrl, parseRow);
//Defining the x length.
const x = xValue((d) => d.petal_width);
//Defining the y length.
const y = yValue((d) => d.sepal_length);

//Defining the markers for different species.
//Red color corresponds to the species "setosa".
//Indigo color corresponds to the species "versicolor".
//Orange color corresponds to the species "verginica".
const marker = (d) => {
    switch(d){
      case 'setosa':
        return 'square';
      case 'versicolor':
        return 'yellow';
      default:
        return 'green';
  }
}

//Getting the marks of for representing the data.

const marks = data.map((d) => ({
  		//Getting the x coordinate and y coordinate from petal_width and sepal_length
      x: xValue((d) => d.petal_width),
      y: yValue((d) => d.sepal_length),
  		//The species of the iris dataset are given using the speciesValue variable declared previously.
      species: speciesValue(d),
  		//Get the color value for the species.
      color: marker(speciesValue(d)),
      rx: rxValue(d),
      ry: ryValue(d),
}));

//Creating a separate function for plotting the scatter plot.
//This is because the function needs to be called in a loop for dynamic plotting.
const plot = scatterPlot()
		//Defining varius attributes of the scatter plot.
    .width(width)
    .height(height)
		//Defining varius attributes of the scatter plot.
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d.petal_width)
    .yValue((d) => d.sepal_length)
		//Defining the margin from the borders.
    .margin({
      top: 20,
      right: 20,
      bottom: 40,
      left: 50,
    })
    .radius(5);

  //Defining the SVG plot.
svg
          .selectAll('circle')
          .data(marks)
          .join('circle')
  //The position of the x and y coordinates.
          .attr('cx', (d) => d.x)
          .attr('cy', (d) => d.y)
  //Fill the marks with corresponding colors.
        	.attr('fill', (d) => d.color)
        	.attr('rx', (d) => (d.rx))
        	.attr('ry', (d) => (d.ry));
//Defining an array named "columns" of different columns in the iris data set.
const columns = [
    'petal_width',
    'sepal_width',
    'petal_length',
    'sepal_length',
  ];
	//Initialising the variable i for looping through the columns.
  let i = 0;
  //Set Interval function is an inbuilt function for calling a function which can be repeated in set intervals of time.
  //Here we are using the setInterval function to call Scatterplot function every 2000 milliseconds.
  setInterval(() => {
    //The value of x axis is being changed according to the array "columns".
    //The values in the array columns are taken in a cyclic manner.
    plot.xValue((d) => d[columns[i % columns.length]]);
    //Get the svg of the chart by calling the function.
    svg.call(plot);
    i++;
  }, 2000);
};
main();



